@@ -1,5 +1,7 @@
 import json
 import re
+import socket
+import time
 from http import HTTPStatus
 from urllib.parse import urljoin, urlencode
 
@@ -24,7 +26,44 @@ def get_header_default(user_agent='ide', content_type='application/json-patch+js
     }
 
 
-class RESTClient:
+def log_params(func):
+    def inner(*args, **kwargs):
+        # log_info = f'{locals()["kwargs"]}.'
+
+        log_info = ' | '.join([f'{key}={value}' for key, value in locals()["kwargs"].items()])
+        if len(args) > 1:
+            log_info += f'UNKNOWN params: {args[1:]}'
+        logger.info(log_info)
+        return func(*args, **kwargs)
+
+    return inner
+
+
+class RESTAssertion:
+    """Parse requests response to get status code."""
+
+    @staticmethod
+    def should_be_success_response_code(response):
+        """200 <= status_code < 400"""
+        assert response.ok, 'Response code is not success.'
+
+    @staticmethod
+    def should_be_ok_response_code(response):
+        """OK, 200. Request fulfilled, document follows"""
+        assert response.status_code == HTTPStatus.OK, 'Response code is not 200.'
+
+    @staticmethod
+    def should_be_not_found_response_code(response):
+        """Nothing matches the given URI"""
+        assert response.status_code == HTTPStatus.NOT_FOUND, 'Response code is not 404.'
+
+    @staticmethod
+    def should_be_bad_request(response):
+        """Bad request syntax or unsupported method"""
+        assert response.status_code == HTTPStatus.BAD_REQUEST, 'Response code is not 400.'
+
+
+class RESTClient(RESTAssertion):
     """ Main class for RestAPI """
 
     def __init__(self,
@@ -96,13 +135,13 @@ class RESTClient:
     def send_request(self,
                      method: str,
                      url: str,
-                     query_params=None,
                      data=None,
                      files=None,
+                     query_params=None,
                      verify=False,
                      custom_header=None,
                      cookies=None,
-                     request_timeout=None):
+                     request_timeout=60):
 
         """
         Send common request
@@ -152,6 +191,12 @@ class RESTClient:
             if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
                 if data and not files:
 
+                    logger.info(f'{method} {link}')
+                    logger.info(f'Headers={self.header}, '
+                                f'Cookies={cookies}, '
+                                f'Timeout={request_timeout}')
+                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
+
                     response = requests.request(method=method,
                                                 url=link,
                                                 auth=self.auth_basic,
@@ -161,15 +206,18 @@ class RESTClient:
                                                 verify=verify,
                                                 timeout=request_timeout)
 
-                    logger.info(f'{method} ' + link)
-                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
-
                 elif files:
                     files_header = self.header.copy()
                     del files_header['Content-Type']
                     if custom_header:
                         files_header = custom_header
 
+                    logger.info(f'{method} {link}')
+                    logger.info(f'Headers={files_header}, '
+                                f'Cookies={cookies}, '
+                                f'Timeout={request_timeout}')
+                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
+
                     response = requests.request(method=method,
                                                 url=link,
                                                 auth=self.auth_basic,
@@ -179,10 +227,13 @@ class RESTClient:
                                                 verify=verify,
                                                 timeout=request_timeout)
 
-                    logger.info(f'{method} ' + link)
+                elif not files and not data:
+                    logger.info(f'{method} {link}')
+                    logger.info(f'Header={self.header}, '
+                                f'Cookies={cookies}, '
+                                f'Timeout={request_timeout}')
                     logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
 
-                elif not files and not data:
                     response = requests.request(method=method,
                                                 url=link,
                                                 auth=self.auth_basic,
@@ -191,18 +242,17 @@ class RESTClient:
                                                 verify=verify,
                                                 timeout=request_timeout)
 
-                    logger.info(f'{method} ' + link)
-                    logger.info('PAYLOAD:\n' + json.dumps(data, indent=4))
-
             # For 'GET', 'HEAD' request
             else:
 
                 # For usual 'GET' without query params
-                header = self.header
-                if custom_header:
-                    header = custom_header
+                header = custom_header if custom_header is not None else self.header
+
+                logger.info(f'{method} {link}')
+                logger.info(f'Header={header}, '
+                            f'Cookies={cookies}, '
+                            f'Timeout={request_timeout}')
 
-                logger.info(f'{method} ' + link)
                 response = requests.request(method=method,
                                             url=link,
                                             auth=self.auth_basic,
@@ -254,27 +304,27 @@ class RESTClient:
     def GET(self,
             url='',
             query_params=None,
-            extend_header=False,
             cookies=None,
-            request_timeout=None):
+            request_timeout=None,
+            header=None):
 
-        return self.send_request('GET',
+        return self.send_request(method='GET',
                                  url=url,
                                  query_params=query_params,
                                  cookies=cookies,
-                                 request_timeout=request_timeout)
+                                 request_timeout=request_timeout,
+                                 custom_header=header)
 
     # noinspection PyPep8Naming
     def POST(self,
              url='',
-             query_params=None,
              data=None,
              file=None,
-             extend_header=False,
+             query_params=None,
              cookies=None,
              request_timeout=None):
 
-        return self.send_request('POST',
+        return self.send_request(method='POST',
                                  url=url,
                                  query_params=query_params,
                                  data=data,
@@ -285,47 +335,81 @@ class RESTClient:
     # noinspection PyPep8Naming
     def PUT(self,
             url='',
-            extend_header=False,
-            custom_header=None,
             data='',
             files='',
-            request_timeout=None,
-            verify=False):
+            query_params=None,
+            cookies=None,
+            request_timeout=None):
 
         header = self.header.copy()
 
         if files:
             del header['Content-Type']
 
-        if custom_header:
-            header = custom_header
-        elif extend_header:
-            header = extend_header
-
-        return requests.request(method='PUT',
-                                url=url,
-                                auth=self.auth_basic,
-                                headers=header,
-                                data=data,
-                                files=files,
-                                verify=verify,
-                                timeout=request_timeout)
+        return self.send_request(method='PUT',
+                                 url=url,
+                                 data=data,
+                                 files=files,
+                                 query_params=query_params,
+                                 cookies=cookies,
+                                 request_timeout=request_timeout)
 
     def DELETE(self,
                url='',
                data='',
                query_params=None,
-               extend_header=False,
                cookies=None,
                request_timeout=None):
 
-        return self.send_request('DELETE',
+        return self.send_request(method='DELETE',
                                  url=url,
                                  query_params=query_params,
                                  data=data,
                                  cookies=cookies,
                                  request_timeout=request_timeout)
 
+    def is_host_available(self, port: int = 0, timeout: int = 5) -> bool:
+        """Check remote host availability using socket and specified port"""
+
+        port_ = port or self.port
+        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as sock:
+            sock.settimeout(timeout)
+            result = sock.connect_ex((self.host, port_))
+            return False if result else True
+
+    def is_service_initialized(self, timeout: int = 20) -> bool:
+        """GET https://{IP}:{PORT}
+
+        :param timeout:
+        :return:
+        """
+
+        try:
+            response = requests.get(self.base_url, verify=False, timeout=timeout)
+            return response.ok
+        except requests.exceptions.ConnectionError:
+            return False
+
+    def wait_service_start(self, timeout: int = 30) -> bool:
+        """Waiting for the service start.
+
+        :param timeout: in sec.
+        :return:
+        """
+
+        timer = 0
+        status = self.is_service_initialized(1)
+
+        while not status:
+            status = self.is_service_initialized(1)
+            timer += 1
+
+            if timer > timeout:
+                print(f'The service was not started within {timeout} seconds.')
+                raise TimeoutError
+            time.sleep(1)
+        return status
+
 
 class AuthToken:
     """Class to get auth token"""
@@ -346,6 +430,8 @@ class AuthToken:
                 return response.json()['token']
             except json.decoder.JSONDecodeError as err:
                 logger.error(err)
+                logger.info(f'Login URL: {self.auth_url}')
+                logger.info(self.client.auth_payload)
                 return 'Cannot get token property: {}'.format(err)
             except TypeError as err:
                 logger.error(err)
@@ -353,27 +439,3 @@ class AuthToken:
                        'Check credentials.\n{}'.format(err)
         else:
             return f'Host {self.client.host} is unreachable. Check service status'
-
-
-class RESTAssertion:
-    """Parse requests response to get status code."""
-
-    @staticmethod
-    def should_be_success_response_code(response):
-        """200 <= status_code < 400"""
-        assert response.ok, 'Response code is not success.'
-
-    @staticmethod
-    def should_be_ok_response_code(response):
-        """OK, 200. Request fulfilled, document follows"""
-        assert response.status_code == HTTPStatus.OK, 'Response code is not 200.'
-
-    @staticmethod
-    def should_be_not_found_response_code(response):
-        """Nothing matches the given URI"""
-        assert response.status_code == HTTPStatus.NOT_FOUND, 'Response code is not 404.'
-
-    @staticmethod
-    def should_be_bad_request(response):
-        """Bad request syntax or unsupported method"""
-        assert response.status_code == HTTPStatus.BAD_REQUEST, 'Response code is not 400.'
